        subroutine getsp(nch,sampr,scale,nfil,iftype,afparam 
     1   ,decl,stcor,orient,chid,cda,cdb,lclkd)
        
!      include 'iounits.inc'
       use iounits_inc
c Station
c       reads info from file to set up filter corrections, conversion
c       of counts to physical units, conversion of elctrode coordinates
c       to an orthogonal (geomagnetic) coordinate system
c
c        inputs required:
c              nch = # of channels
c              dr =  sampling rate (needed only if no system parameter
c                                 file is provided.... for gds data only
c                               the routine will assume a two pole low pass
c                               filter with a time constant equal to the 
c                               nyquist period.)
c               stname =   character string giving station name; routine
c                         trys to open a system parameter file named spstname

c>>>>>>>>>>>> 28 Feb, 1991: afparam is changed to an aray of
c       character*80 variables to allow more general filter/response
c         parameters (e.g., names of EMI calibration files)

cf 7.6.95 Magson Magnetometer 10 sec filter added

cnew	Add two filter identifiers for coefficients of analog filters
cnew	provided in form of simple tables, one as real and imaginary part
cnew	and one as amplitude and phase, both as functions of period.
cnew	Within the table files all header lines should be marked with one of the
cnew	standard comment characters such as #, %, $, &, *
cnew	AP	is the identifier for the amplitude/phase table type
cnew	RI	is the identifier for the rela/imaginary table type

ccc      21 Aug 2002: afparam is changed to character*120
ccc      16 Dec 2002: afparam is changed to character*256
ccc      16 August, 2006: Egbert cleans up, merges in two new filter types
ccc         from other versions: DT (time delay for channel) and PZ (pole-zero)

!        include 'params2.inc'
        use params2_inc
        use SD_param       !TH:  just for magnetic declination
        
        real scale(nchmx),orient(2,*),stcor(2),decl
        integer iftype(nfilmax,*),nfil(*),lc

 !       character*256 afparam(nfilmax,nchmx)
        character*256 afparam(nfilmax,6)
        character*2 ftype               
        character*6 chid(*)
        character*20 cname
        character*20 ctemp
        logical lclkd

ccc     Open system paramter file
	print *,sp_unit
        open(unit =sp_unit, file = cfsp,status='old',err=10)
        go to 11
10      continue
c       try stripping off one character (remove a, b, c ... suffix)
c       and open again
        lc = irlong(cfsp,80)
        cfsp = cfsp(1:lc-1)//'.sp'
        open(unit =sp_unit, file = cfsp,status='old',err=90)
11      continue

c       station name ... does not appear to be used after reading
c          ... length might be a problem
        read(sp_unit,'(a20)',err=100) cname
	print *,cname
c       deblank station name
        do k=1,20
         if(cname(1:1).eq.' ')then
           cname(1:19)=cname(2:20)
           cname(20:20)=' '
         endif 
        enddo

c       position of station; latitude and longitude; fractions of degrees
c       are assumed expressed as decimals (i.e. 45:30:00 = 45.5000)
c       make longitude (assumed degrees E) positive
        read(sp_unit,*,err=100) stcor      
        if(stcor(2).lt. 0.0) stcor(2) = 360.0+stcor(2)

c       magnetic declination of station
        read(sp_unit,*,err=100) decl
        if (decl .eq. 0) decl = MagnetDecl    ! use calculated magnetic declination
c       number of data channels
        read(sp_unit,*,err=100) nch
cTH.....if channel 6 was not diabled, dirty tricks applied        
        nch_enable = nch  
        if (nch .eq. 6) then
            nch_enable = nch
            nch = 5
        endif

c       sampling rate; assumed in seconds
        read(sp_unit,*,err=100) sampr

c       clock drift corrections; assume clock time = tc; actual time = t
c       these are related by tc = t + (cda + cdb*t); assumes t in seconds
c       these are never used now!
        read(sp_unit,*) cda,cdb
        lclkd = ( abs(cdb) .gt. 1.0e-20)

c       write station info to standard output
        write (6,22) cname,nch, 1./sampr, stcor, decl 
 22     format(/,'Station:',a9,3x,'# of channels:',i3,3x,' Dt:',f8.2,
     &    'Hz',/,'Lat./Long.:',2f7.2,3x,' Declination:',f6.2,/)
        write(6,23)
 23     format('    CHID    ORIENT.   TILT    SCALE          FILTERS')

c       get azimuths, filter parameters, etc for each of
        do i = 1,nch
c          channel name
           read(sp_unit,'(a6)',err=100) chid(i) 
           if( (chid(i)(1:1).eq.'e') .or. (chid(i)(1:1).eq.'E') ) then
c             ELECTRIC FIELD CHANNEL:
c             electrode line length, 
c               +  azimuth (deg.  e of geomangnetic north of line
c                       from neg. to pos. electrode), tilt
c               +  amplifier gain
              read(sp_unit,*,err=100) r,orient(1,i),orient(2,i),ampg
              r = r/1000.     ! Gary defines electrode length in km, KMS in m

c             count conversion ( = mV/count = 1/sensitivity);
c               + number of filters to correct for for electric field channels
!              read(sp_unit,*,err=100) scale(i),nfil(i)  ! Gary's version
!              scale(i) = scale(i)/(ampg*r)              ! gary's version
              read(sp_unit,*,err=100) gain_factor,nfil(i)   ! TH: kms gains
              scale(i) = 1./gain_factor/(ampg*r)
              
           else
c             MAGNETIC FIELD CHANNEL (or something else ...):
c             azimuth, tilt 
              read(sp_unit,*) orient(1,i),orient(2,i)
c             count conversion ( = nt/count = 1/sensitivity);
c             number of filters to correct for for magnetic field channels
!TH              read(sp_unit,*) scale(i),nfil(i)
              read(sp_unit,*) gain_factor,nfil(i)
              scale(i) = 1./gain_factor
           end if

c          read analogue filter/system response info
           ctemp = ' '
           do j = 1,nfil(i)
c             FOR EACH FILTER read 2 character string identifying filter type
              read(sp_unit,'(a2)',err=100) ftype
              ctemp = ctemp(1:iclong(ctemp,20))//' '//ftype//' '
              read(sp_unit,'(a)',err=100) afparam(j,i)
!              write(0,*) 'ftype:  ',ftype
!              write(0,*) 'afparam:',afparam(j,i)
              
c             Assign integer index for each filter in array iftype
              if((ftype.eq.'l1').or.(ftype.eq.'L1')) then
c ..................one pole low pass filter
c                    2 real parameters:
                 iftype(j,i) = 5
              else if((ftype.eq.'l2').or.(ftype.eq.'L2')) then
c ..................two pole low pass filter
c                    3 real parameters:
                 iftype(j,i) = 4
              else if((ftype.eq.'h1').or.(ftype.eq.'H1')) then
c ..................one pole hi pass filter
c                    2 real parameters:
                 iftype(j,i) = 2
              else if((ftype.eq.'h2').or.(ftype.eq.'H2')) then
c ..................two pole hi pass filter
c                    3 real parameters:
                 iftype(j,i) = 3
              else if((ftype.eq.'bc').or.(ftype.eq.'BC')) then
c .................."box car" average
c                    3 real parameters:
                 iftype(j,i) = 6
              else if((ftype.eq.'te').or.(ftype.eq.'TE')) then
c ...................EMI electric field calibration table
c                         Three parameters:
c                         file name, mode of operation, gain
                 iftype(j,i) = 7
              else if((ftype.eq.'tb').or.(ftype.eq.'TB')) then
c ...................EMI mag field calibration table
c                         One parameter:
c                         file name
                 iftype(j,i) = 8
              else if((ftype.eq.'pz').or.(ftype.eq.'PZ')) then
c....................Poles and zeros representation of system response TF
                 iftype(j,i) = 10

C-PDAS  
C-PDAS  31/8/96 these are the old analytical transfer functions
C-PDAS  new ones, based on the true calibration files have names
C-PDAS  PA/pa and 05 for PAB and MFS05, respect., see below
 
	      else if ((ftype.eq.'pd').or.(ftype.eq.'PD')) then
c....................PDAS 200 Hz low pass filter
c                    all parameters in FUNCTION AFCOR
		iftype(j,i) = 11
	      else if ((ftype.eq.'mx').or.(ftype.eq.'MX')) then
c....................METRONIX GMS05 coil
c                    all parameters in FUNCTION AFCOR
	      iftype(j,i) = 12

C-PDAS end
C  LMT 
	      else if ((ftype.eq.'RA').or.(ftype.eq.'ra')) then
c....................10 sec low pass of RAP telluric
c                    all parameters in FUNCTION AFCOR
		iftype(j,i) = 13
	      else if ((ftype.eq.'MA').or.(ftype.eq.'ma')) then
c....................MAGSON transfer function (5 sec)
c                    all parameters in FUNCTION AFCOR
	        iftype(j,i) = 14
	      else if ((ftype.eq.'R2').or.(ftype.eq.'r2')) then
c....................120 sec low pass of RAP telluric
c		     all parameters in FUNKTION AFCOR
	        iftype(j,i) = 15
	      else if ((ftype.eq.'M1').or.(ftype.eq.'m1')) then
c....................MAGSON transfer function (10 sec)
c                    all parameters in FUNCTION AFCOR
	        iftype(j,i) = 16
C  END LMT

C-PDAS  new transfer functions based on calibration data files
	      else if ((ftype.eq.'PA').or.(ftype.eq.'pa')) then
C....................PAB KMT preamp, data in calibration file
	        iftype(j,i) = 17
	      else if (ftype.eq.'05') then
C..................MFS05 transfer functions from calibration data file
	        iftype(j,i) = 18
C-PDAS   new end

             elseif (ftype.eq.'UC') then
ccc             uncallibrated coil i omega response
                iftype(j,i) = 97
             elseif (ftype.eq.'AP') then
                iftype(j,i) = 98
             elseif (ftype.eq.'RI') then
                iftype(j,i) = 99
cBooker:     filter to correct timing offsets in individual channels
             elseif ((ftype.eq.'DT').or.(ftype.eq.'dt')) then
                iftype(j,i) = 101
cBooker:     filter to correct timing offsets in individual channels

             else 
                 iftype(j,i) = 9
                 write(6,*) 'filter type',ftype,' unknown'
              end if
            
           enddo  

c          write filter info to standard output
           write (6,'(i2,2x,a6,5x,2f7.2,e12.5,i3,1x,a9)')i,chid(i),
     &       orient(1,i),orient(2,i),scale(i),nfil(i),
     &       ctemp(1:iclong(ctemp,20))

       enddo
       close(sp_unit)
CTH....case channel 6 was not disabled       
       if (nch_enable .eq. 6) then
           nch = 6
       endif
       
       return

ccc    Error conditions:

90     print*,'error: system parameter file ',cfsp,' not found'
       stop
100    print*,'error reading system parameter file ',cfsp
       stop

       end



        subroutine getspkms(nch_mt,sampr,scale,nfil,iftype,
     1   afparam ,decl,stcor,orient,chid,cda,cdb,lclkd)
        
!      include 'iounits.inc'
       use iounits_inc
c Station
c       reads info from binary header of KMS820 to set up filter corrections, conversion
c       of counts to physical units, conversion of elctrode coordinates
c       to an orthogonal (geomagnetic) coordinate system
c
c        inputs required:
c              nch = # of channels
c              dr =  sampling rate (needed only if no system parameter
c                                 file is provided.... for gds data only
c                               the routine will assume a two pole low pass
c                               filter with a time constant equal to the 
c                               nyquist period.)
c               stname =   character string giving station name; routine
c                         trys to open a system parameter file named spstname

c>>>>>>>>>>>> 28 Feb, 1991: afparam is changed to an aray of
ccc      16 Dec 2002: afparam is changed to character*256
ccc      16 August, 2006: Egbert cleans up, merges in two new filter types
ccc         from other versions: DT (time delay for channel) and PZ (pole-zero)

!        include 'params2.inc'
        use params2_inc
        use SD_param
        USE SD_file_parameters
        
        real scale(nchmx),orient(2,*),stcor(2),decl  !these obnoxious lazy * dimensions really suck!
        integer iftype(nfilmax,*),nfil(*),lc

    !    character*256 afparam(nfilmax,nchmx)
        character*256 afparam(nfilmax,6)
        character*2 ftype               
        character*6 chid(*)
        character*1 glon_c,glat_c
        character*40 cname
        character*40 ctemp
        logical lclkd
        integer i_stop
        character*256 ccc
        
!     statement definition for SD parameter        
      INTEGER*2, EXTERNAL :: in16Endian
      INTEGER*4, EXTERNAL :: in32Endian
      INTEGER*8, EXTERNAL :: in64Endian
      INTEGER*4, EXTERNAL :: unsign2
      INTEGER*8, EXTERNAL :: unsign4
      
      CHARACTER(LEN=6), DIMENSION(6)  :: chid_sd
      CHARACTER(LEN=64)               :: char_string, char_string_rev
      character(len=32)               :: string,cstring
      CHARACTER(LEN=1), DIMENSION(8)  :: char_string_SD
      INTEGER                         :: nchan,nch_enable,nch_xx,nch_mt
      INTEGER, DIMENSION(6)           :: ich_gain1
      INTEGER, DIMENSION(6)           :: ich_gain2
      REAL, DIMENSION(6)              :: ch_filt_LPcutoff
      INTEGER                         :: i, ii, nbytes4, len_string
      INTEGER                         :: num, k, kk
      INTEGER, DIMENSION(6)           :: ch_enable,ich_enable,ich_field
      INTEGER, DIMENSION(6)           :: ch_sens_id, ich_rank
      REAL, DIMENSION(6)              :: ch_orient, ch_length
      CHARACTER(LEN=2)                :: c2num
      CHARACTER(LEN=4)                :: c4num
      CHARACTER(LEN=6)                :: c6num
      character(len=256)              :: dummy
      LOGICAL                         :: l_wrsp, l_HZ, l_unsort
      REAL                            :: A0                 ! amplitude factor at DC of LP bessel
      INTEGER                         :: nzeroes, npoles    ! # zeroes & poles of LP bessel
      COMPLEX, DIMENSION(10)          :: zeroes, poles
        
!     excutable statements
ccc     Open system paramter file
        if (l_wrsp) then
            open(unit=19,file='wrsp.txt')
        endif    

c       station name is taken from filename
c          ... length might be a problem
!        read(sp_unit,'(a80)',err=100) cname
!        cname = adjustl(flnam)
!        print *,'cname',cname

c       position of station; latitude and longitude; fractions of degrees
c       are assumed expressed as decimals (i.e. 45:30:00 = 45.5000)
c       make longitude (assumed degrees E) positive
        
        cstring = SD_H%GPS_pos_info
        READ(cstring,110) idum1,glat_m,idum2,glon_m   ! splitting needed
  110   FORMAT(I2,F11.6,2x,i3,F12.6)   !corrected BRL -changed 3x,i2 to 2x,i3
        READ(cstring,115) glat_c,glon_c
  115   FORMAT(T13,A1,T28,A1)
!        hat zahlen richtig gelesen aber nicht die beiden characters - makes as much sense as the rest of this crap!
!        READ(cstring,110) idum1,glat_m,glat_c,idum2,glon_m,glon_c
!  110   FORMAT(I2,F11.6,1x,a1,1x,i2,F12.6,1x,a1)        
        glat_d = float(idum1) + glat_m/60.
        if(glat_c .eq. 'S') glat_d = -glat_d
        glon_d = float(idum2) + glon_m/60.
        if(glon_c .eq. 'W') glon_d = -glon_d
        stcor(1) = glat_d
        stcor(2) = glon_d
        print *,'stcor',stcor(1),stcor(2)
        if(stcor(2).lt. 0.0) stcor(2) = 360.0+stcor(2)

c       magnetic declination of station
        string(1:21) = SD_H%GPS_time_info(4:24)     ! date
        read(string,'(i2,i2,f6.3,1x,i2,1x,i2,1x,i4)')
     &  ihr,imin,ss,iday, imo,iyr
!       print *,'ihr,imin,ss,iday,imo,iyr',ihr,imin,ss,iday,imo,iyr
        yeardate = float(iyr) + float(imo)/12.
		call calc_MagnDecl (glat_d,glon_d,yeardate,decl)
cew        call calc_magnDecl (glat_d,glon_d,yeardate,decl)
        print *,'from GETSP: magnetic declination=',decl
c       number of data channels
!        read(sp_unit,*,err=100) nch  



c       sampling rate; assumed in seconds
        sampfreq = unsign4(in32Endian(SD_H%sampling_frequency))
        sampr = 1./sampfreq
!        read(sp_unit,*,err=100) sampr

c       clock drift corrections; assume clock time = tc; actual time = t
c       these are related by tc = t + (cda + cdb*t); assumes t in seconds
c       these are never used now!
        cda = 0.
        cdb = 0.
        lclkd = ( abs(cdb) .gt. 1.0e-20)

!TH:    

!     get control_word, find which channels are enabled

      print *,'--------------begin header info ---------------------'   
      nbytes4 = 4
      char_string_SD(1:4) = SD_H%AD_ctrl_word(1:4)
      call get_BitSeq(nbytes4,char_string_SD,char_string,len_string)

      do i=1,len_string   ! reverse is easier to compare with bit position
         char_string_rev(len_string-i+1:len_string-i+1)=char_string(i:i)
      enddo

   !   char_string_rev(4:5) = '00'

      nchan = 0
      do i=1,6
          SELECT CASE (char_string_rev(i:i))
          CASE ("0") 
             ch_enable(i) = 0
             write(6,'(A5,i2,A12)') "Chan(",i," )  disabled"
          CASE ("1")
             nchan = nchan + 1 
             ch_enable(i) = 1
             ich_enable(nchan) = i
             write(6,'(A5,i2,A11)') "Chan(",i," )  enabled"
          END SELECT
      enddo   
      nch_enable = nchan
      nch_mt = nch_enable
!      print *,'number of channel = ',nchan
!      do i=1,nchan
!          print *,' i, ich_enable(i)',i, ich_enable(i), ch_enable(ich_enable(i))
!      enddo
!    identify channel
      do i=1,nchan
          ii = ich_enable(i)
          num = ichar(SD_H%sensor_ID(ich_enable(i)))     ! only enabled values in list
          ch_orient(ii) = 
     &float(unsign2(in16Endian(SD_H%sensor_orientation(ich_enable(i)))))
          ch_length(ii) = 
     &   float(unsign2(in16Endian(SD_H%sensor_distance(ich_enable(i)))))
          ch_sens_id(ii) = num
          SELECT CASE (num)
          CASE (2,7)
             chid_sd(ii) = 'E'
          CASE (1,3,4,5,6)
             chid_sd(ii) = 'H'
          CASE DEFAULT
             chid_sd(ii) = '?'
             write (6,*) 'channel no.',i,'is not complete:',chid_sd(ii)
             write (6,*) 'enter E or H'
             read (5,'(A)') chid_sd(ii)(1:1)
          END SELECT            
          chid_sd(ii)(2:2) = SD_H%sensor_component(ich_enable(i))
      enddo
   15 continue   ! repeat chid settings      
      
      l_HZ = .false.       ! flag for Hz, to distinuish between MT with 4 sensor or 5 sensors
      l_unsort = .false.   ! flag for case: no sorting possible, because of nonunique components
      nch_mt = 0
      nch_xx = 0
      do i=1,nchan                             ! check which magnetic field sensors
          ii = ich_enable(i)
          if (ii .gt. 0) then
          c2num(1:2) = chid_sd(ii)
          SELECT CASE (c2num)
          CASE ('HX')
              nch_mt = nch_mt + 1
              ich_field(ii) = 1
              ich_rank(ich_field(ii)) = ii !ch_enable(i)
          CASE ('HY')
              nch_mt = nch_mt + 1
              ich_field(i) = 2    
              ich_rank(ich_field(ii)) = ii !ch_enable(i)
          CASE ('HZ')
              nch_mt = nch_mt + 1
              ich_field(ii) = 3
              l_HZ = .true.
              ich_rank(ich_field(ii)) = ii !ch_enable(i)
          CASE ('EX')
              nch_mt = nch_mt + 1
              ich_field(ii) = 4      ! position if  Hz
              ich_rank(ich_field(ii)) = ii !ch_enable(i)
          CASE ('EY')
              nch_mt = nch_mt + 1
              ich_field(ii) = 5      ! position if  Hz
              ich_rank(ich_field(ii)) = ii !ch_enable(i)
          CASE DEFAULT
              nch_xx = nch_xx + 1
              l_unsort = .true.     ! field components are not well defined
              print *,'l_unsort and nch_xx:',l_unsort,nch_xx
          END SELECT
          end if
      enddo 
      print *,'nch_mt',nch_mt
      
      IF (l_unsort)  then
          write(6,*) '***** chid settings not complete, enter manually'
          write(6,*) 'correct is: HX, HY, HZ, EX, EY'
          write(6,*) '#channnels',nchan
          do i=1,nchan
              ii = ich_enable(i)
              write(6,*) 'ch. no.:',i,'  chid: ',chid_sd(ii)
              read (5,'(A)') c2num
              if(c2num(1:1).NE.' ') READ(c2num,*) chid_sd(ii)
!             convert to capital letters              
              if (chid_sd(ii)(1:1) .eq. 'e') chid_sd(ii)(1:1) = 'E'
              if (chid_sd(ii)(1:1) .eq. 'h') chid_sd(ii)(1:1) = 'H'
              if (chid_sd(ii)(2:2) .eq. 'x') chid_sd(ii)(2:2) = 'X'
              if (chid_sd(ii)(2:2) .eq. 'y') chid_sd(ii)(2:2) = 'Y'
              if (chid_sd(ii)(2:2) .eq. 'z') chid_sd(ii)(2:2) = 'Z'
              
          enddo
      endif        
      if (l_unsort .AND. nchan .eq. 5) goto 15
      
      
      IF (.not. l_HZ) THEN      
          DO i=1,nchan                             ! define channel numbers for E-field
              ii = ich_enable(i)
          IF (ich_field(ii) == 4 .OR. ich_field(ii) == 5) THEN
              ich_field(ii) = ich_field(ii) - 1
          ENDIF
          ENDDO
      ENDIF    
      IF (l_unsort) THEN   ! no sorting of channels, because field components are not precise
          DO i=1,nchan
              ich_field(i) = i
              ich_rank(i)  = i
          ENDDO
      ENDIF        
!     get channel setting status: gain and filter           
      char_string_SD(1:8) = SD_H%ch_setting
      call get_BitSeq(8,char_string_SD,char_string,len_string)

      do i=1,len_string   ! reverse is easier to compare with bit position
         char_string_rev(len_string-i+1:len_string-i+1)=char_string(i:i)
      enddo

!
!     gain1+2 and LP setting checked for all 6 channels
!
      DO i=1,6                  ! gain 1 MUL setting for all channels
          ii = 1 + (i-1)*2      ! bit 1:11 step 2
          SELECT CASE (char_string_rev(ii:ii+1))
          CASE ("00") 
             ich_gain1(i) = 1
          CASE ("10")           ! reversed bit order
             ich_gain1(i) = 1
          CASE ("01")           ! reversed bit order
             ich_gain1(i) = 8
          CASE ("11")
             ich_gain1(i) = 40
          END SELECT
      ENDDO              
      DO i=1,6                  ! gain2 SEL  for all channels ch1 to ch6
          ii = 13 + (i-1)*2     ! bit 13:23 step 2     
          SELECT CASE (char_string_rev(ii:ii+1))
          CASE ("00") 
             ich_gain2(i) = 1
          CASE ("10") ! reversed
             ich_gain2(i) = 4
          CASE ("01")
             ich_gain2(i) = 16
          CASE ("11")
             ich_gain2(i) = 64
          END SELECT
      ENDDO
      DO i=1,6               ! get cut off frequencies of LP filter for all channels
          ii = 25 + (i-1)*2      ! bit 25:35 step 2
          SELECT CASE (char_string_rev(ii:ii+1))
              CASE ("00") 
                  ch_filt_LPcutoff(i) = 20000.
              CASE ("10") ! reversed
                  ch_filt_LPcutoff(i) = 1000.
              CASE ("01") ! reversed
                  ch_filt_LPcutoff(i) = 10.
              CASE ("11")
                  ch_filt_LPcutoff(i) = 0.
          END SELECT
      ENDDO    
      
      !what the F%$$!! is going on here -this means the reference channel gets no orientation data

      
c      if (nch_mt .lt. 4) return


         
      write(6,*) 
     & '************ channel setting from SD card *******************'
      write(6,*) 
     & ' i, enable, fieldNo., chid, orientation, E-field length (m)'
      do i=1,nchan
          c2num(1:2) = chid_sd(i)
          IF (c2num(1:1) == 'H') THEN
              write(6,210) i,ich_enable(i),ich_field(i),chid_sd(i),
     &         ch_orient(i)
          ELSEIF (c2num(1:1) == 'E') THEN
              write(6,220) i,ich_enable(i),ich_field(i),chid_sd(i),
     &         ch_orient(i),ch_length(i)
          ELSE    
              write(6,220) i,ich_enable(i),ich_field(i),chid_sd(i),
     &         ch_orient(i),ch_length(i)
          ENDIF
! 210 FORMAT(1x,i2,3x,i3,6x,a6,2x,f6.0,4x,f6.0)              
! 220 FORMAT(1x,i2,3x,i3,6x,a6,2x,f6.0)     
      enddo         

!      write(6,*) '************ NO SORTING: chid is not exact for MT ****************'
      write(6,*) 
     & '************ sorted channels SP parameter setting *************'
      write(6,*) 
     &  ' i, enable, fieldNo.,  chid, orientation, E-field length (m)'
      do i=1,nchan
          ii = ich_rank(i)
          IF (ii .GT. 0) THEN 
          c2num(1:2) = chid_sd(ii)
          IF (c2num(1:1) == 'H') THEN
              write(6,210) i,ich_enable(ii),ich_field(ii),chid_sd(ii),
     &         ch_orient(ii)
          ELSEIF (c2num(1:1) == 'E') THEN
              i_stop=0
              write(6,220) i,ich_enable(ii),ich_field(ii),chid_sd(ii),
     &         ch_orient(ii), ch_length(ii)
          ELSE    
              write(6,220) i,ich_enable(ii),ich_field(ii),chid_sd(ii),
     &         ch_orient(ii),ch_length(ii)
                                  
          ENDIF
         ENDIF
  210 FORMAT(1x,i2,3x,i3,6x,i3,6x,a6,2x,f6.0)     
  220 FORMAT(1x,i2,3x,i3,6x,i3,6x,a6,2x,f6.0,8x,f6.0)              
      enddo         
      write(6,*) '*** gain 1:MUL and 2:SEL and LP filter setting ***'
      write(6,*) 'ch, enable, gain1_MUL,gain2_SEL,cut off Freq. (Hz)'
  240 FORMAT(1x,i2,3x,i3,6x,i3,6x,i3,6x,F10.0)
      do i=1,nchan
          ii = ich_enable(i)
          write(6,240) 
     &     i, ii,ich_gain1(ii),ich_gain2(ii),ch_filt_LPcutoff(ii)
      enddo              
!      do i=1,nchan
!          print *,'i, chid',i,chid_sd(i), ch_sens_id(i),ch_orient(i),ch_length(i)
!      enddo      

!     sensor id for magnetic calibration files
      c4num = '.rsp'
      c4num = adjustl(c4num)
      do i=1,nchan
          ii = ich_enable(i)
          num = ichar(SD_H%sensor_ID(ich_enable(ii)))     ! only enabled values in list
          !num=ich_enable(i) !changed BRL
          afparam(1,i) = 'none'
          SELECT CASE (num)
          case (3)
             ctemp(1:4)=adjustl(SD_H%sensor_SN(ii))
             afparam(1,i) ='LEMI-118_'//ctemp(1:4)//c4num
          CASE (4)
             ctemp(1:4)=adjustl(SD_H%sensor_SN(ii))  
             afparam(1,i) ='LEMI-120'//'_'//ctemp(1:4)//c4num !corrected BRL
          CASE (5)
!            lemi coils have only one rsp file for all coils    
              !WRONG - they don't in our case, they also have a serial number
             ctemp(1:4)=adjustl(SD_H%sensor_SN(ii)(1:4))  
             ccc='LEMI-120'//'_'//ctemp(1:4)//c4num !corrected BRL
             afparam(1,i) =ccc
             i_stop=0
             
          CASE (6)
             ctemp(1:4)=adjustl(SD_H%sensor_SN(ii))
             afparam(1,i) ='LEMI-121'//c4num
          END SELECT            
      enddo


!     get filter settings and sacle factors:       

        write(6,23)
 23     format('    CHID    ORIENT.   TILT    SCALE          FILTERS')
      do i=1,nchan
          ii = ich_enable(i)
          chid(i) = chid_sd(ii)
          if (chid(i)(1:1).eq.'e' .or. chid(i)(1:1).eq.'E') THEN
          
              scale(i) = 1./float(ich_gain1(ii)*ich_gain2(ii))
              scale(i) = scale(i)/ch_length(ii)*1000  ! lenghth unit km Gary
              orient(1,i) = ch_orient(ii)
              orient(2,i) = 0.    ! tilt
              
              if (ch_filt_LPcutoff(ii) == 0) then
!                 no filter              
                  ctemp = '  '
                  nfil(i) = 0
              else    
                  nfil(i) = 1
                  iftype(1,i) = 10    ! PZ filter
                  ctemp = 'PZ'
                  call get_pzLPbessel(ch_filt_LPcutoff(ii),
     &                A0,npoles,poles,nzeroes,zeroes)
              
                  write(afparam(1,i),*) nzeroes,npoles,A0,
     &              (zeroes(kk),kk=1,nzeroes),(poles(k),k=1,npoles)
              endif
          else if (chid(i)(1:1).eq.'h' .or. chid(i)(1:1).eq.'H') THEN
              ctemp = trim(ctemp)
              ctemp = '     '
              scale(i) = 1./float(ich_gain1(ii)*ich_gain2(ii))
              orient(1,i) = ch_orient(ii)
              orient(2,i) = 0.    ! tilt
              nfil(i) = 0
              
              if (afparam(1,i)(1:4) /= 'none') then  !changed /= to == BRL
                  nfil(i) = nfil(i) + 1
                  iftype(1,i) = 98    ! AP filter type
                  ctemp = 'AP'
              endif
              if (ch_filt_LPcutoff(ii) > 0) then
!                 low pass filters 
                  nfil(i) = nfil(i) + 1
                  iftype(2,i) = 10    ! PZ filter
                  ctemp = trim(ctemp)//' PZ'
                  call get_pzLPbessel(ch_filt_LPcutoff(ii),A0,npoles,
     &              poles,nzeroes,zeroes)
                  write(afparam(2,i),*) nzeroes,npoles,A0,
     &             (zeroes(kk),kk=1,nzeroes),(poles(k),k=1,npoles)
              endif
          else
              write(6,*) 'SP parameters not correct and complete'
              write(6,*) 'create an ASCII SP-file and run w. option -sp'
              stop
          endif    

!          write(6,*) ' filter info to standard output'
           write (6,'(i2,2x,a6,5x,2f7.2,e12.5,i3,1x,a9)')i,chid(i), 
     &       orient(1,i),orient(2,i),scale(i),nfil(i), ctemp
            
          
      enddo    
      write(6,*) 'calibration info: (AP filename, PZ poles & zeroes)'
      do i=1,nchan
          print *,'afparam:',trim(afparam(1,i))
      enddo    

      print *,'---------------end header info--------------------------'

      return
      end subroutine getspkms
      
      
      SUBROUTINE get_pzLPbessel(Fc,A0,npoles,poles,nzeroes,zeroes)
!     get poles and zeros of kms LP filter: 20kHz (Bessel 4), 1 kHz (Bessel 4), 10 Hz Bessel 2
      IMPLICIT NONE
      REAL, INTENT(in)                    :: Fc                ! cut-off freq in Hz
      REAL, INTENT(out)                   :: A0                ! amplitude at DC
      INTEGER, INTENT(out)                :: npoles, nzeroes   ! # poles and zeroes
      COMPLEX, DIMENSION(10), INTENT(out) :: poles, zeroes     ! comples poles and zeroes
!     local variables
      REAL, PARAMETER           :: twopi= 6.283185307
      REAL                      :: wc,a1,b1,a2,b2               ! filter parameters
      COMPLEX                   :: discrim,s1,s2,s3,s4,caux     ! zeroes
      INTEGER                   :: i, k
      CHARACTER(LEN=256)        :: dummy
!     first executable statement
      nzeroes = 0    ! only LP, so no zeroes
      do i=1,10
          zeroes = cmplx(0.,0.)
          poles = cmplx(0.,0.)
      enddo    
      IF (Fc == 20000 .OR. Fc == 1000.) THEN
          a1 = 1.3397         ! Bessel filter param. order 4 (Tietze & Schenk chap. 14)
          b1 = 0.4889
          a2 = 0.7743
          b2 = 0.3890
          wc = twopi*Fc
          caux = a1*a1- 4.*b1
          discrim = SQRT(caux)
          s1 = (-a1 + discrim)/2./b1
          s2 = (-a1 - discrim)/2./b1
          caux = a2*a2 - 4.*b2
          discrim = SQRT(caux)
          s3 = (-a2 + discrim)/2./b2
          s4 = (-a2 - discrim)/2./b2
          A0 = wc**4/b1/b2
          npoles = 4
          poles(1) = s1*wc
          poles(2) = s2*wc
          poles(3) = s3*wc
          poles(4) = s4*wc
      ELSE IF (Fc == 10.) THEN
          a1 = 1.3397         ! Bessel filter param. order 4 (Tietze & Schenk chap. 14)
          b1 = 0.4889
          wc = twopi*Fc
          caux = a1*a1 - 4.*b1
          discrim = SQRT(caux)
          s1 = (-a1 + discrim)/2./b1
          s2 = (-a1 - discrim)/2./b1
          A0 = wc**2/b1
          npoles = 2
          poles(1) = s1*wc
          poles(2) = s2*wc
      ELSE
          A0 = 1.
          npoles = 0
          nzeroes = 0
      ENDIF       
      
      caux = a0
      do i=1,npoles
          caux = caux/poles(i)
      enddo
!      print *,'amp',caux,'   fc',fc    
      
!      write(dummy,*) nzeroes,npoles,A0,(zeroes(k),k=1,nzeroes),(poles(i),i=1,npoles)
              
 !     print '(A)',adjustl(dummy)
      
      RETURN
      END SUBROUTINE get_pzLPbessel
    
      SUBROUTINE get_BitSeq(Nbytes,cstring,char_BitSeq,len_BitSeq)
      IMPLICIT NONE
      INTEGER, INTENT(IN)                  :: Nbytes
      CHARACTER*1,DIMENSION(*), INTENT(IN) :: cstring
      CHARACTER(LEN=64), INTENT(OUT)       :: char_BitSeq
      INTEGER*4                            :: len_BitSeq
!   local variables    
      INTEGER*4         :: Nterm,i,k
      CHARACTER(LEN=32)                :: c32Bit
      INTEGER*4, DIMENSION(32)         :: i32Bit
      CHARACTER(LEN=8)                 :: c8Bit
      LOGICAL, PARAMETER               :: Lcheck = .false.    ! .true. control output on screen
      LOGICAL LfirstBit
!   first executable statement    
!   convert each char*1-element to 8-bit sequence and store in long sequence
      DO i=1,Nbytes
        Nterm = ichar(cstring(i))
        call get_c1BitSeq(Nterm,c32Bit,i32Bit,Lcheck)
        c8Bit(1:8) = c32Bit(25:32)
        char_BitSeq((i-1)*8+1:(i-1)*8+8) = c8Bit(1:8)
      ENDDO    
      len_BitSeq = 8*Nbytes
      RETURN
      END SUBROUTINE get_BitSeq
        
   
      SUBROUTINE get_c1BitSeq(NARG,cbitseq,ibitseq,LCHECK)
      IMPLICIT NONE
      INTEGER, INTENT(IN)    :: NARG
      LOGICAL, INTENT(IN)    :: LCHECK
      CHARACTER(LEN=32),INTENT(OUT)         :: cBitSeq
      INTEGER*4, DIMENSION(32), INTENT(OUT) :: iBitSeq
!     local variables   
      INTEGER IB,i,N
      LOGICAL LfirstBit
    
      N = NARG
      LfirstBit = .false.
      IF (N .LT. 0) THEN
        LfirstBit = .true.
        N = NOT(N)
      ENDIF
    
      DO i=32,1,-1
        IB = MOD(N,2)
        IF (IB == 1) THEN
           iBitSeq(i)= IB
           cBitSeq(i:i) = "1"
        ELSE
           iBitSeq(i) = IB
           cBitSeq(i:i) = "0"
        ENDIF
!        print *,'i,n,ib,cBitSeq',i,n,ib,cBitSeq
        N = N/2
      ENDDO
      IF (LfirstBit) THEN
        DO I=1,32        ! take complementary
           IF (iBitSeq(i) .EQ. 1) THEN
               iBitSeq(i) = 0
               cBitSeq(i:i) = "0"
            ELSE
               iBitSeq(i) = 1
               cBitSeq(i:i) = "1"
            ENDIF
        ENDDO
      ENDIF   
      IF (LCHECK) THEN
         print *,'** begin control output on screen from get_c1BitSeq *'
         print *,'input number: ',NARG
         print *,
     &    '32-bit sequence: position,integer array & character string:'
         print '(32I2)',(i,i=32,1,-1)
         print '(32I2)',(iBitSeq(i),i=1,32)
         print *,cBitSeq
!        2*31 = - 2147483648 negative for integer*4
!        so just summing up the bits
         DO i=1,32
             N = N + iBitSeq(i)*2**(32-i)
!             print *,i,N,iBitSeq(i)
         ENDDO
         print *,'convert bit to integer number:',N
         print *,'   '
         print *,'*** end control output from get_c1BitSeq ***'
      ENDIF        
     
      RETURN
      END SUBROUTINE get_c1BitSeq
   
      

